"""
Author:         David Szybala
Website:        www.davidszybala.com
Alt Website:    www.twistedgerbil.com
Program:        grid_lines.py
Date:           11/05/14
Description:    This script will alter the size/divisions the home grid.
                Insert the grid units as inches and the script will convert
                them to the scene unit.
Installation:   Place this file in your maya scripts directory
                (e.g. ..\My Documents\maya\scripts) then run the following
                python commands:
                    import grid_lines
                    grid_lines.main()
"""
import maya.cmds as cmds
from re import search


class GridLines(object):
    def __init__(self):
        self.units = ['mm', 'cm', 'm', 'km', 'in', 'ft', 'yd', 'mi']
        self.current_unit = cmds.currentUnit(query=True, linear=True)
        self.grid_size_spinner = None
        self.unit_dropdown = None
        self.grid_minor_spinner = None
        self.grid_subdivs_spinner = None
        self.status_label = None

    @staticmethod
    def get_unit_conversion(unit_value, from_unit, to_unit):
        """Returns the float value of converting 'unit_value' from 'from_unit'
        to 'to_unit' """
        conversion = cmds.convertUnit(str(unit_value), fromUnit=from_unit,
                                      toUnit=to_unit)

        if type(conversion) != float:
            conversion = search(r'^[0-9]*', conversion).group()
            conversion = float(conversion)

        return conversion

    def set_grid_button_pressed(self, *args):
        """Sets the grid spacing for the scene"""
        grid_size = self.get_unit_conversion(cmds.floatField(self.grid_size_spinner, query=True, value=True),
                                             cmds.optionMenu(self.unit_dropdown, query=True, value=True),
                                             cmds.currentUnit(query=True, linear=True))
        grid_spacing = self.get_unit_conversion(cmds.floatField(self.grid_minor_spinner, query=True, value=True),
                                                cmds.optionMenu(self.unit_dropdown, query=True, value=True),
                                                cmds.currentUnit(query=True, linear=True))
        cmds.grid(toggle=True, size=grid_size, spacing=grid_spacing,
                  divisions=cmds.intField(self.grid_subdivs_spinner, query=True, value=True))

        message = 'INFO: Grid size set to '
        message += str(cmds.floatField(self.grid_size_spinner, query=True, value=True))
        message += cmds.optionMenu(self.unit_dropdown, query=True, value=True)
        cmds.text(self.status_label, edit=True, label=message)

    def setup_ui(self):
        """Creates the UI"""
        if cmds.window('gridLinesWindow', exists=True):
            cmds.deleteUI('gridLinesWindow')
        
        grid_lines_window = cmds.window('gridLinesWindow', title='Grid Lines',
                                        width=215, height=195, minimizeButton=False,
                                        maximizeButton=False, sizeable=True,
                                        resizeToFitChildren=True)
        cmds.columnLayout('windowLayout')
        cmds.frameLayout('unitFrame', label='Units', collapsable=False, marginHeight=5,
                         marginWidth=5)
        cmds.text(label=('Scene Units: ' + self.current_unit),
                  width=200, align='left')
        cmds.setParent('..')  # end frameLayout
        cmds.frameLayout('gridFrame', label='Grid', collapsable=False, marginHeight=5,
                         marginWidth=5)
        cmds.rowColumnLayout('sizeLayout', width=200, numberOfColumns=3, columnAlign=[1, 'left'],
                             columnWidth=[(1, 100), (2, 50), (3, 50)],
                             columnAttach=[1, 'left', 0])
        cmds.text(label='Length/Width:', align='left')
        self.grid_size_spinner = cmds.floatField('sizeSpinner', width=45, minValue=2,
                                                 maxValue=9999, precision=2,
                                                 step=1.0, value=120.0)
        self.unit_dropdown = cmds.optionMenu('unitDropdown', label='')
        cmds.menuItem(label='mm')
        cmds.menuItem(label='cm')
        cmds.menuItem(label='m')
        cmds.menuItem(label='km')
        cmds.menuItem(label='in')
        cmds.menuItem(label='ft')
        cmds.menuItem(label='yd')
        cmds.menuItem(label='mi')
        cmds.text(label='Grid Lines Every:', align='left')
        self.grid_minor_spinner = cmds.floatField('minorSpinner', width=45, minValue=2,
                                                  maxValue=9999, precision=2,
                                                  step=1, value=60)
        cmds.text(label='')
        cmds.text(label='Subdivisions:', align='left')
        self.grid_subdivs_spinner = cmds.intField('subdivSpinner', width=45, minValue=2,
                                                  maxValue=99, step=1, value=5)
        cmds.setParent('..')  # end rowColumnLayout
        cmds.setParent('..')  # end frameLayout
        cmds.columnLayout('buttonLayout', width=205, columnAttach=['left', 5], rowSpacing=5)
        cmds.text(label='', height=5)
        cmds.button('gridButton', width=200, label='Set Grid Lines Now',
                    command=self.set_grid_button_pressed)
        self.status_label = cmds.text('statusLabel', label='INFO: Standing by...', align='left')
        cmds.setParent('..')  # end columnLayout
        cmds.setParent('..')  # end columnLayout

        if self.units.count(self.current_unit) > 0:
            cmds.optionMenu(self.unit_dropdown, edit=True,
                            select=(self.units.index(self.current_unit)) + 1)

        cmds.showWindow(grid_lines_window)


def main():
    """Main entry point. Opens the UI"""
    gl = GridLines()
    gl.setup_ui()


if __name__ == '__main__':
    main()
